@@ -10,7 +10,7 @@
 #include "qpdf_pdftopdf.h"
 
 // Use: content.append(debug_box(pe.sub,xpos,ypos));
-static std::string debug_box(const PageRect &box,float xshift,float yshift) // {{{ 
+static std::string debug_box(const PageRect &box,float xshift,float yshift) // {{{
 {
   return std::string("q 1 w 0.1 G\n ")+
          QUtil::double_to_string(box.left+xshift)+" "+QUtil::double_to_string(box.bottom+yshift)+" m  "+
@@ -51,7 +51,7 @@ QPDF_PDFTOPDF_PageHandle::QPDF_PDFTOPDF_PageHandle(QPDF *pdf,float width,float h
   // xobjects: later (in get())
   content.assign("q\n");  // TODO? different/not needed
 
-  page=pdf->makeIndirectObject(page); // stores *pdf 
+  page=pdf->makeIndirectObject(page); // stores *pdf
 }
 // }}}
 
@@ -147,7 +147,7 @@ void QPDF_PDFTOPDF_PageHandle::add_border_rect(const PageRect &_rect,BorderType
   boxcmd+="Q\n";
 
 // if (!isExisting()) {
-//   // TODO: only after 
+//   // TODO: only after
 //   return;
 // }
 
@@ -180,7 +180,7 @@ void QPDF_PDFTOPDF_PageHandle::add_subpage(const std::shared_ptr<PDFTOPDF_PageHa
   std::string xoname="/X"+QUtil::int_to_string((qsub->no!=-1)?qsub->no:++no);
   if (crop) {
     PageRect pg=qsub->getRect(),tmp=*crop;
-    // we need to fix a too small cropbox. 
+    // we need to fix a too small cropbox.
     tmp.width=tmp.right-tmp.left;
     tmp.height=tmp.top-tmp.bottom;
     tmp.rotate_move(-getRotate(qsub->page),tmp.width,tmp.height); // TODO TODO (pg.width? / unneeded?)
@@ -192,7 +192,7 @@ void QPDF_PDFTOPDF_PageHandle::add_subpage(const std::shared_ptr<PDFTOPDF_PageHa
     if (pg.height<tmp.height) {
       pg.top=pg.bottom+tmp.height;
     }
-    
+
     PageRect rect=ungetRect(pg,*qsub,ROT_0,qsub->page);
 
     qsub->page.replaceKey("/TrimBox",makeBox(rect.left,rect.bottom,rect.right,rect.top));
@@ -218,7 +218,7 @@ void QPDF_PDFTOPDF_PageHandle::add_subpage(const std::shared_ptr<PDFTOPDF_PageHa
   content.append(xoname+" Do\n");
   content.append("Q\n");
 }
-// }}} 
+// }}}
 
 void QPDF_PDFTOPDF_PageHandle::mirror() // {{{
 {
@@ -228,9 +228,13 @@ void QPDF_PDFTOPDF_PageHandle::mirror() // {{{
     // need to wrap in XObject to keep patterns correct
     // TODO? refactor into internal ..._subpage fn ?
     std::string xoname="/X"+QUtil::int_to_string(no);
-    xobjs[xoname]=makeXObject(page.getOwningQPDF(),page);
 
-    *this=QPDF_PDFTOPDF_PageHandle(page.getOwningQPDF(),orig.width,orig.height);
+    QPDFObjectHandle subpage=get();  // this->page, with rotation
+
+    // replace all our data
+    *this=QPDF_PDFTOPDF_PageHandle(subpage.getOwningQPDF(),orig.width,orig.height);
+
+    xobjs[xoname]=makeXObject(subpage.getOwningQPDF(),subpage); // we can only now set this->xobjs
 
 //    content.append(std::string("1 0 0 1 0 0 cm\n  ");
     content.append(xoname+" Do\n");
@@ -240,7 +244,7 @@ void QPDF_PDFTOPDF_PageHandle::mirror() // {{{
 
   static const char *pre="%pdftopdf cm\n";
   // Note: we don't change (TODO need to?) the media box
-  std::string mrcmd("-1 0 0 1 "+ 
+  std::string mrcmd("-1 0 0 1 "+
                     QUtil::double_to_string(orig.right)+" 0 cm\n");
 
   content.insert(0,std::string(pre)+mrcmd);
@@ -253,6 +257,94 @@ void QPDF_PDFTOPDF_PageHandle::rotate(Rotation rot) // {{{
 }
 // }}}
 
+void QPDF_PDFTOPDF_PageHandle::add_label(const PageRect &_rect, const std::string label) // {{{
+{
+  assert(isExisting());
+
+  PageRect rect = ungetRect (_rect, *this, rotation, page);
+
+  assert (rect.left <= rect.right);
+  assert (rect.bottom <= rect.top);
+
+  // TODO: Only add in the font once, not once per page.
+  QPDFObjectHandle font = page.getOwningQPDF()->makeIndirectObject (
+    QPDFObjectHandle::parse(
+      "<<"
+      " /Type /Font"
+      " /Subtype /Type1"
+      " /Name /pagelabel-font"
+      " /BaseFont /Helvetica" // TODO: support UTF-8 labels?
+      ">>"));
+  QPDFObjectHandle resources = page.getKey ("/Resources");
+  QPDFObjectHandle rfont = resources.getKey ("/Font");
+  rfont.replaceKey ("/pagelabel-font", font);
+
+  double margin = 2.25;
+  double height = 12;
+
+  std::string boxcmd = "q\n";
+
+  // White filled rectangle (top)
+  boxcmd += "  1 1 1 rg\n";
+  boxcmd += "  " + QUtil::double_to_string(rect.left + margin) + " " +
+		   QUtil::double_to_string(rect.top - height - 2 * margin) + " " +
+		   QUtil::double_to_string(rect.right - rect.left - 2 * margin) + " " +
+		   QUtil::double_to_string(height + 2 * margin) + " re f\n";
+
+  // White filled rectangle (bottom)
+  boxcmd += "  " + QUtil::double_to_string(rect.left + margin) + " " +
+		   QUtil::double_to_string(rect.bottom + height + margin) + " " +
+		   QUtil::double_to_string(rect.right - rect.left - 2 * margin) + " " +
+		   QUtil::double_to_string(height + 2 * margin) + " re f\n";
+
+  // Black outline (top)
+  boxcmd += "  0 0 0 RG\n";
+  boxcmd += "  " + QUtil::double_to_string(rect.left + margin) + " " +
+		   QUtil::double_to_string(rect.top - height - 2 * margin) + " " +
+		   QUtil::double_to_string(rect.right - rect.left - 2 * margin) + " " +
+		   QUtil::double_to_string(height + 2 * margin) + " re S\n";
+
+  // Black outline (bottom)
+  boxcmd += "  " + QUtil::double_to_string(rect.left + margin) + " " +
+		   QUtil::double_to_string(rect.bottom + height + margin) + " " +
+		   QUtil::double_to_string(rect.right - rect.left - 2 * margin) + " " +
+		   QUtil::double_to_string(height + 2 * margin) + " re S\n";
+
+  // Black text (top)
+  boxcmd += "  0 0 0 rg\n";
+  boxcmd += "  BT\n";
+  boxcmd += "  /pagelabel-font 12 Tf\n";
+  boxcmd += "  " + QUtil::double_to_string(rect.left + 2 * margin) + " " +
+		   QUtil::double_to_string(rect.top - height - margin) + " Td\n";
+  boxcmd += "  (" + label + ") Tj\n";
+  boxcmd += "  ET\n";
+
+  // Black text (bottom)
+  boxcmd += "  BT\n";
+  boxcmd += "  /pagelabel-font 12 Tf\n";
+  boxcmd += "  " + QUtil::double_to_string(rect.left + 2 * margin) + " " +
+		   QUtil::double_to_string(rect.bottom + height + 2 * margin) + " Td\n";
+  boxcmd += "  (" + label + ") Tj\n";
+  boxcmd += "  ET\n";
+
+  boxcmd += "Q\n";
+
+  assert(page.getOwningQPDF()); // existing pages are always indirect
+  static const char *pre="%pdftopdf q\n"
+                         "q\n",
+                    *post="%pdftopdf Q\n"
+                          "Q\n";
+
+  QPDFObjectHandle stm1=QPDFObjectHandle::newStream(page.getOwningQPDF(),
+						    std::string(pre)),
+                   stm2=QPDFObjectHandle::newStream(page.getOwningQPDF(),
+						    std::string(post) + boxcmd);
+
+  page.addPageContents(stm1,true); // before
+  page.addPageContents(stm2,false); // after
+}
+// }}}
+
 void QPDF_PDFTOPDF_PageHandle::debug(const PageRect &rect,float xpos,float ypos) // {{{
 {
   assert(!isExisting());
@@ -260,7 +352,6 @@ void QPDF_PDFTOPDF_PageHandle::debug(const PageRect &rect,float xpos,float ypos)
 }
 // }}}
 
-
 void QPDF_PDFTOPDF_Processor::closeFile() // {{{
 {
   pdf.reset();
@@ -273,6 +364,7 @@ void QPDF_PDFTOPDF_Processor::error(const char *fmt,...) // {{{
   va_list ap;
 
   va_start(ap,fmt);
+  fputs("ERROR: ",stderr);
   vfprintf(stderr,fmt,ap);
   fputs("\n",stderr);
   va_end(ap);
@@ -419,7 +511,7 @@ void QPDF_PDFTOPDF_Processor::add_page(std::shared_ptr<PDFTOPDF_PageHandle> page
 void QPDF_PDFTOPDF_Processor::multiply(int copies,bool collate) // {{{
 {
   assert(pdf);
-  assert(copies>0); 
+  assert(copies>0);
 
   std::vector<QPDFObjectHandle> pages=pdf->getAllPages(); // need copy
   const int len=pages.size();
@@ -439,7 +531,36 @@ void QPDF_PDFTOPDF_Processor::multiply(int copies,bool collate) // {{{
   }
 }
 // }}}
- 
+
+// TODO? elsewhere?
+void QPDF_PDFTOPDF_Processor::autoRotateAll(bool dst_lscape,Rotation normal_landscape) // {{{
+{
+  assert(pdf);
+
+  const int len=orig_pages.size();
+  for (int iA=0;iA<len;iA++) {
+    QPDFObjectHandle page=orig_pages[iA];
+
+    Rotation src_rot=getRotate(page);
+
+    // copy'n'paste from QPDF_PDFTOPDF_PageHandle::getRect
+    PageRect ret=getBoxAsRect(getTrimBox(page));
+//    ret.translate(-ret.left,-ret.bottom);
+    ret.rotate_move(src_rot,ret.width,ret.height);
+//    ret.scale(getUserUnit(page));
+
+    const bool src_lscape=(ret.width>ret.height);
+    if (src_lscape!=dst_lscape) {
+      Rotation rotation=normal_landscape;
+      // TODO? other rotation direction, e.g. if (src_rot==ROT_0)&&(param.orientation==ROT_270) ... etc.
+      // rotation=ROT_270;
+
+      page.replaceOrRemoveKey("/Rotate",makeRotate(src_rot+rotation));
+    }
+  }
+}
+// }}}
+
 #include "qpdf_cm.h"
 
 // TODO
@@ -451,7 +572,7 @@ void QPDF_PDFTOPDF_Processor::addCM(const char *defaulticc,const char *outputicc
     return; // nothing to do
   }
 
-  QPDFObjectHandle srcicc=setDefaultICC(*pdf,defaulticc);
+  QPDFObjectHandle srcicc=setDefaultICC(*pdf,defaulticc); // TODO? rename to putDefaultICC?
   addDefaultRGB(*pdf,srcicc);
 
   addOutputIntent(*pdf,outputicc);
@@ -464,9 +585,10 @@ void QPDF_PDFTOPDF_Processor::addCM(const char *defaulticc,const char *outputicc
 void QPDF_PDFTOPDF_Processor::setComments(const std::vector<std::string> &comments) // {{{
 {
   extraheader.clear();
-  for (auto &it : comments) {
-    assert(it.at(0)=='%');
-    extraheader.append(it);
+  const int len=comments.size();
+  for (int iA=0;iA<len;iA++) {
+    assert(comments[iA].at(0)=='%');
+    extraheader.append(comments[iA]);
     extraheader.push_back('\n');
   }
 }
